/*------------------------------------------------------------------------------
 *       Filename:  ar.h
 *
 *    Description:  Header file for activity recognition
 *
 *        Version:  2.0
 *        Created:  05/07/2012 06:46:23 PM
 *       Revision:  none
 *       Compiler:  gcc
 *
 *        Authors:  Narayanan C Krishnan, ckn@eecs.wsu.edu
 *                  Diane J. Cook, cook@eecs.wsu.edu
 *   Organization:  Washington State University
------------------------------------------------------------------------------*/

#ifndef AR_H
#define AR_H

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include "sys/stat.h"
#include "svm.h"

#define MAXSTR 80          // Maximum string size
#define TRAIN 0
#define TEST 1
#define CROSSVAL 2
#define NONE 0
#define TIMEWEIGHT 1
#define MUTUALINFO 2
#define MUTUALINFO_TIME 3
#define NUMWINDOWS 4
#define MAXWINDOW 30
#define LN 2.30265441084134
#define MIN 0.000000000001
#define KAPPA 0.125
#define TRUE 1
#define FALSE 0


char **filenames = NULL;   // names of the training data files
char *treefname = NULL;    // name of the file that contains the tree structure
char *modelfname = NULL;   // filename for storing / loading the activity model
char *translatefilename;   // file containing the translations of sensorids
char ***translate;         // mapping of the sensor ids
int Mode = TRAIN;          // 0=train, 1=test
int Weight = NONE;         // 0=none, 1=time, 2=mutual info, 3=time+mutual info
int Context = FALSE;       // 0=no context, 1=include previous activity
int OutputLevel = 1;       // Ranges from 0 to 2
int NumFiles = 0;          // number of training data files
int hierarchical = FALSE;  // whether the labels form a hierarchy
int NumTranslate;          // number of mappings

// AR variables
char **sensornames = NULL; // sensor names
char **activitynames = NULL; // activity names
char ***tree;              // hsvm hierarchy structure
int **adindices;           // indices of samples for each activity.
int *numdataperactivity;   // number of samples per activity
int NumActivities = 0;     // total number of activities
int NumData;               // total number of data samples
int NumSensors = 0;        // total number of sensors
int TreeDepth = 1;         // depth of the hierarchy tree
int *NumSiblings;          // number of siblings at every node of the tree
int windata[MAXWINDOW][3];
int wincnt = 0;
int PrevActivity = 0;
int CurActivity = 0;
int OtherActivityIndex;
float **data;              // feature vector matrix derived from sensor data
float **MI;                // mutual information for sensor pairs

struct svm_model **models;        // models for online recognition
struct svm_model **contextmodels; // context models for online recognition


int NumFeatures;            // feature vector size

// Dynamic window variables
int **actwind;
int *factwind;
int **senact;
int *fsenact;

// SVM parameters
double C = 10;
double GAMMA = 0.01;

void ReadConfig(char *name);  // Read the configuration file
void PrintParms();            // Print the program parameter values
void ComputeProbActWind();    // Compute the activity probability distributions
                              // for alternative sensor windows
void ComputeProbSenAct();     // Compute the activity probability distribution
                              // for each sensor
void ComputeMI();             // Compute mutual information for sensor pairs
void ReadData();              // Read the data from the individual files and
                              // transform data into feature vectors
                              // Update current window, create feature vector
                              // Add the data index to an activity
void AddDataToActivity(char *name, int count);
void ReadTree();              // Read the activity hierarchy from file
void DefaultTree();           // Create the flat model
void LearnActivityModels();   // Learn the SVM activity models
void TrainSVM(int *labels, int numlabel, char *modelfilename, int probability);
void TestActivityModels();    // Test activity models using data from a file
void ReadTranslate();         // Read the sensor id translations
void LoadModels(int level);   // Load SVM models level from a file
void SaveParams();            // Save the training parameters
void FreeMemory();            // Free allocated memory

int FindActivity(char *name); // Find the index of an activity from
                              // an activity list
int FindSensor(char *name);   // Find the index of a sensor from a sensor list
int ComputeFeature(char *time, char *sensorid, float *tempdata);
int ComputeSeconds(char *time); // Compute seconds past midnight
int ComputeHour(char *time);  // Compute hour of day based on 24 hour clock
int GetWindow(char *name);    // Compute best window size for the given sensor
	                      // Test the SVM model using data files
int TestSVM(int derivecontext, float *tempdata, float *datacontext,
            int numcontext);
int TestOnline(float *data);  // Test each data point separately

#endif // AR_H
